/*
@(#)File:           $RCSfile: sqlsetjmp.c,v $
@(#)Version:        $Revision: 1.1 $
@(#)Last changed:   $Date: 2017/01/11 01:32:38 $
@(#)Purpose:        SQLCMD setjmp and longjmp handling
@(#)Author:         J Leffler
@(#)Copyright:      (C) JLSS 2017
@(#)Product:        SQLCMD Version 91.02 (2018-10-28)
*/

/*TABSTOP=4*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <setjmp.h>
#include "jlsstools.h"
#include "sqlerr.h"

#ifdef HAVE_SIGLONGJMP
static sigjmp_buf jb;
#else
static jmp_buf    jb;
#endif /* HAVE_SIGLONGJMP */

#ifndef lint
/* Prevent over-aggressive optimizers from eliminating ID string */
extern const char jlss_id_sqlsetjmp_c[];
const char jlss_id_sqlsetjmp_c[] = "@(#)$Id: sqlsetjmp.c,v 1.1 2017/01/11 01:32:38 jleffler Exp $";
#endif /* lint */

void  sql_setjmp(void (*process)(int argc, char **argv), int argc, char **argv,
                 void (*cleanup)(int status), int status)
{
#ifdef HAVE_SIGLONGJMP
    if (sigsetjmp(jb, 1) != 0)
#else
    if (setjmp(jb) != 0)
#endif /* HAVE_SIGLONGJMP */
        (*cleanup)(status);
    (*process)(argc, argv);
}

void sql_longjmp(int status)
{
#ifdef HAVE_SIGLONGJMP
    siglongjmp(jb, status);
#else
    longjmp(jb, status);
#endif /* HAVE_SIGLONGJMP */
}
